@@ -95,6 +95,9 @@ translate_message2(error(resource_error(stack), Context)) -->
 translate_message2(error(resource_error(tripwire(Wire, Context)), _)) -->
     !,
     tripwire_message(Wire, Context).
+translate_message2(error(existence_error(reset, Ball), SWI)) -->
+    swi_location(SWI),
+    tabling_existence_error(Ball, SWI).
 translate_message2(error(ISO, SWI)) -->
     swi_location(SWI),
     term_message(ISO),
@@ -325,6 +328,25 @@ domain(range(Low,High)) -->
 domain(Domain) -->
     ['`~w\''-[Domain] ].
 
+%!  tabling_existence_error(+Ball, +Context)//
+%
+%   Called on invalid shift/1  calls.  Track   those  that  result  from
+%   tabling errors.
+
+tabling_existence_error(Ball, Context) -->
+    { table_shift_ball(Ball) },
+    [ 'Tabling dependency error' ],
+    swi_extra(Context).
+
+table_shift_ball(dependency(_Head)).
+table_shift_ball(dependency(_Skeleton, _Trie, _Mono)).
+table_shift_ball(call_info(_Skeleton, _Status)).
+table_shift_ball(call_info(_GenSkeleton, _Skeleton, _Status)).
+
+%!  dwim_predicates(+PI, -Dwims)
+%
+%   Find related predicate indicators.
+
 dwim_predicates(Module:Name/_Arity, Dwims) :-
     !,
     findall(Dwim, dwim_predicate(Module:Name, Dwim), Dwims).
