@@ -63,7 +63,7 @@
             '$tbl_answer'/4,            % +Trie, -Return, -ModeArgs, -Delay
 
             '$wrap_tabled'/2,		% :Head, +Mode
-            '$moded_wrap_tabled'/4,	% :Head, +ModeTest, +Variant, +Moded
+            '$moded_wrap_tabled'/5,	% :Head, +Opts, +ModeTest, +Varnt, +Moded
             '$wfs_call'/2,              % :Goal, -Delays
 
             '$set_table_wrappers'/1,    % :Head
@@ -631,8 +631,8 @@ delim(Skeleton, Worker, WorkList, Delays) :-
 %   As start_tabling/2, but in addition separates the data stored in the
 %   answer trie in the Variant and ModeArgs.
 
-'$moded_wrap_tabled'(Head, ModeTest, WrapperNoModes, ModeArgs) :-
-    '$set_predicate_attribute'(Head, tabled, true),
+'$moded_wrap_tabled'(Head, Options, ModeTest, WrapperNoModes, ModeArgs) :-
+    set_pattributes(Head, Options),
     '$wrap_predicate'(Head, table, Closure, Wrapped,
                       (   ModeTest,
                           start_moded_tabling(Closure, Head, Wrapped,
@@ -641,7 +641,21 @@ delim(Skeleton, Worker, WorkList, Delays) :-
 
 
 start_moded_tabling(Closure, Wrapper, Worker, WrapperNoModes, ModeArgs) :-
-    '$tbl_moded_variant_table'(Closure, WrapperNoModes, Trie, Status, Skeleton),
+    '$tbl_moded_variant_table'(Closure, WrapperNoModes, Trie,
+                               Status, Skeleton, IsMono),
+    (   IsMono == true
+    ->  shift(dependency(Skeleton/ModeArgs, Trie, Mono)),
+        (   Mono == true
+        ->  tdebug(monotonic, 'Monotonic new answer: ~p', [Skeleton])
+        ;   start_moded_tabling_2(Closure, Wrapper, Worker, ModeArgs,
+                                  Trie, Status, Skeleton)
+        )
+    ;   start_moded_tabling_2(Closure, Wrapper, Worker, ModeArgs,
+                              Trie, Status, Skeleton)
+    ).
+
+start_moded_tabling_2(_Closure, Wrapper, Worker, ModeArgs,
+                      Trie, Status, Skeleton) :-
     (   Status == complete
     ->  moded_gen_answer(Trie, Skeleton, ModeArgs)
     ;   functor(Status, fresh, 2)
@@ -1109,20 +1123,24 @@ current_table(Variant, Trie) :-
     current_table_lookup(Variant, Trie),
     !.
 
-current_table_gen(Variant, Trie) :-
+current_table_gen(M:Variant, Trie) :-
     '$tbl_local_variant_table'(VariantTrie),
-    trie_gen(VariantTrie, Variant, Trie).
-current_table_gen(Variant, Trie) :-
+    trie_gen(VariantTrie, M:NonModed, Trie),
+    M:'$table_mode'(Variant, NonModed, _Moded).
+current_table_gen(M:Variant, Trie) :-
     '$tbl_global_variant_table'(VariantTrie),
-    trie_gen(VariantTrie, Variant, Trie),
-    \+ '$tbl_table_status'(Trie, fresh). % shared tables are not destroyed
+    trie_gen(VariantTrie, NonModed, Trie),
+    \+ '$tbl_table_status'(Trie, fresh), % shared tables are not destroyed
+    M:'$table_mode'(Variant, NonModed, _Moded).
 
-current_table_lookup(Variant, Trie) :-
+current_table_lookup(M:Variant, Trie) :-
+    M:'$table_mode'(Variant, NonModed, _Moded),
     '$tbl_local_variant_table'(VariantTrie),
-    trie_lookup(VariantTrie, Variant, Trie).
-current_table_lookup(Variant, Trie) :-
+    trie_lookup(VariantTrie, M:NonModed, Trie).
+current_table_lookup(M:Variant, Trie) :-
+    M:'$table_mode'(Variant, NonModed, _Moded),
     '$tbl_global_variant_table'(VariantTrie),
-    trie_lookup(VariantTrie, Variant, Trie),
+    trie_lookup(VariantTrie, NonModed, Trie),
     \+ '$tbl_table_status'(Trie, fresh).
 
 ct_generate(M:Variant) :-
@@ -1200,7 +1218,7 @@ wrappers(ModeDirectedSpec, Module, Opts) -->
       (   ModeTest == true
       ->  WrapClause = '$wrap_tabled'(Module:Head, Opts),
           TVariant = Head
-      ;   WrapClause = '$moded_wrap_tabled'(Module:Head, ModeTest,
+      ;   WrapClause = '$moded_wrap_tabled'(Module:Head, Opts, ModeTest,
                                             Module:Variant, Moded),
           TVariant = Variant
       )
@@ -1816,11 +1834,7 @@ try_reeval(ATrie, Goal, Return) :-
     nb_current('$tbl_reeval', true),
     !,
     tdebug(reeval, 'Nested re-evaluation for ~p', [ATrie]),
-    '$tbl_reeval_prepare_top'(ATrie, Clause),
-    (   nonvar(Clause)
-    ->  trie_gen_compiled(Clause, Return)
-    ;   call(Goal)
-    ).
+    do_reeval(ATrie, Goal, Return).
 try_reeval(ATrie, Goal, Return) :-
     tdebug(reeval, 'Planning reeval for ~p', [ATrie]),
     findall(Path, false_path(ATrie, Path), Paths0),
@@ -1832,12 +1846,21 @@ try_reeval(ATrie, Goal, Return) :-
                   tdebug(reeval, '  Re-eval complete path: ~p', [Path]))),
     reeval_paths(Dynamic, ATrie),
     reeval_paths(Complete, ATrie),
+    do_reeval(ATrie, Goal, Return).
+
+do_reeval(ATrie, Goal, Return) :-
     '$tbl_reeval_prepare_top'(ATrie, Clause),
-    (   nonvar(Clause)
+    (   Clause == 0                          % complete and answer subsumption
+    ->  '$tbl_table_status'(ATrie, _Status, M:Variant, Return),
+        M:'$table_mode'(Goal0, Variant, ModeArgs),
+        Goal = M:Goal0,
+        moded_gen_answer(ATrie, Return, ModeArgs)
+    ;   nonvar(Clause)                       % complete
     ->  trie_gen_compiled(Clause, Return)
-    ;   call(Goal)
+    ;   call(Goal)                           % actually re-evaluate
     ).
 
+
 split_paths([], [], []).
 split_paths([[_|Path]|T], DT, [Path|CT]) :-
     split_paths(T, DT, CT).
@@ -1919,16 +1942,18 @@ is_invalid(ATrie) :-
 %       outer one it will complete before we continue.
 
 reeval_node(ATrie) :-
-    '$tbl_reeval_prepare'(ATrie, Variant),
+    '$tbl_reeval_prepare'(ATrie, M:Variant),
     !,
-    tdebug(reeval, 'Re-evaluating ~p', [Variant]),
+    M:'$table_mode'(Goal0, Variant, _Moded),
+    Goal = M:Goal0,
+    tdebug(reeval, 'Re-evaluating ~p', [Goal]),
     (   '$idg_reset_current',
         setup_call_cleanup(
             nb_setval('$tbl_reeval', true),
-            ignore(Variant),                    % assumes local scheduling
+            ignore(Goal),                    % assumes local scheduling
             nb_delete('$tbl_reeval')),
         fail
-    ;   tdebug(reeval, 'Re-evaluated ~p', [Variant])
+    ;   tdebug(reeval, 'Re-evaluated ~p', [Goal])
     ).
 reeval_node(ATrie) :-
     '$mono_reeval_prepare'(ATrie, Size),
