@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker and Richard O'Keefe
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2016, University of Amsterdam
+    Copyright (c)  2002-2020, University of Amsterdam
                               VU University Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -35,6 +36,7 @@
 
 :- module(lists,
         [ member/2,                     % ?X, ?List
+          memberchk/2,                  % ?X, ?List
           append/2,                     % +ListOfLists, -List
           append/3,                     % ?A, ?B, ?AB
           prefix/2,                     % ?Part, ?Whole
@@ -54,6 +56,7 @@
           reverse/2,                    % +List, -Reversed
           permutation/2,                % ?List, ?Permutation
           flatten/2,                    % +Nested, -Flat
+          clumped/2,                    % +Items,-Pairs
 
                                         % Ordered operations
           max_member/2,                 % -Max, +List
@@ -73,8 +76,9 @@
           subset/2,                     % +SubSet, +Set
           subtract/3                    % +Set, +Delete, -Remaining
         ]).
-:- use_module(library(error)).
-:- use_module(library(pairs)).
+:- autoload(library(error),[must_be/2]).
+:- autoload(library(pairs),[pairs_keys/2]).
+
 
 :- set_prolog_flag(generate_debug_info, false).
 
@@ -492,6 +496,38 @@ flatten([Hd|Tl], Tail, List) :-
 flatten(NonList, Tl, [NonList|Tl]).
 
 
+		 /*******************************
+		 *            CLUMPS		*
+		 *******************************/
+
+%!  clumped(+Items, -Pairs)
+%
+%   Pairs is a list  of  `Item-Count`   pairs  that  represents the _run
+%   length encoding_ of Items.  For example:
+%
+%   ```
+%   ?- clumped([a,a,b,a,a,a,a,c,c,c], R).
+%   R = [a-2, b-1, a-4, c-3].
+%   ```
+%
+%   @compat SICStus
+
+clumped(Items, Counts) :-
+    clump(Items, Counts).
+
+clump([], []).
+clump([H|T0], [H-C|T]) :-
+    ccount(T0, H, T1, 1, C),
+    clump(T1, T).
+
+ccount([H|T0], E, T, C0, C) :-
+    E == H,
+    !,
+    C1 is C0+1,
+    ccount(T0, E, T, C1, C).
+ccount(List, _, List, C, C).
+
+
                  /*******************************
                  *       ORDER OPERATIONS       *
                  *******************************/
