@@ -50,11 +50,15 @@
             predicate_sort_key/2,               % +PI, -Key
 
             is_control_goal/1,                  % @Term
-            is_predicate_indicator/1            % @Term
+            is_predicate_indicator/1,           % @Term
+
+            body_term_calls/2                   % :BodyTerm, -Goal
           ]).
 :- autoload(library(error),[must_be/2, instantiation_error/1]).
 :- autoload(library(lists),[append/3]).
 
+:- meta_predicate
+    body_term_calls(:, -).
 
 :- multifile
     user:prolog_predicate_name/2.
@@ -298,3 +302,78 @@ is_control_goal((_->_)).
 is_control_goal((_|_)).
 is_control_goal((_*->_)).
 is_control_goal(\+(_)).
+
+%!  body_term_calls(:BodyTerm, -Goal) is nondet.
+%
+%   True when BodyTerm calls Goal.  This   predicate  looks into control
+%   structures as well as meta predicates based on predicate_property/2.
+
+body_term_calls(M:Body, Calls) :-
+    body_term_calls(Body, M, M, Calls).
+
+body_term_calls(Var, M, C, Calls) :-
+    var(Var),
+    !,
+    qualify(M, C, Var, Calls).
+body_term_calls(M:Goal, _, C, Calls) :-
+    !,
+    body_term_calls(Goal, M, C, Calls).
+body_term_calls(Goal, M, C, Calls) :-
+    qualify(M, C, Goal, Calls).
+body_term_calls((A,B), M, C, Calls) :-
+    !,
+    (   body_term_calls(A, M, C, Calls)
+    ;   body_term_calls(B, M, C, Calls)
+    ).
+body_term_calls((A;B), M, C, Calls) :-
+    !,
+    (   body_term_calls(A, M, C, Calls)
+    ;   body_term_calls(B, M, C, Calls)
+    ).
+body_term_calls((A->B), M, C, Calls) :-
+    !,
+    (   body_term_calls(A, M, C, Calls)
+    ;   body_term_calls(B, M, C, Calls)
+    ).
+body_term_calls((A*->B), M, C, Calls) :-
+    !,
+    (   body_term_calls(A, M, C, Calls)
+    ;   body_term_calls(B, M, C, Calls)
+    ).
+body_term_calls(\+ A, M, C, Calls) :-
+    !,
+    body_term_calls(A, M, C, Calls).
+body_term_calls(Goal, M, C, Calls) :-
+
+    predicate_property(M:Goal, meta_predicate(Spec)),
+    !,
+    arg(I, Spec, SArg),
+    arg(I, Goal, GArg),
+    meta_calls(SArg, GArg, Call0),
+    body_term_calls(Call0, M, C, Calls).
+
+meta_calls(0, Goal, Goal) :-
+    !.
+meta_calls(I, Goal0, Goal) :-
+    integer(I),
+    !,
+    length(Extra, I),
+    extend_goal(Goal0, Extra, Goal).
+meta_calls(//, Goal0, Goal) :-
+    extend_goal(Goal0, [_,_], Goal).
+meta_calls(^, Goal0, Goal) :-
+    !,
+    strip_existential(Goal0, Goal).
+
+strip_existential(Var, Var) :-
+    var(Var),
+    !.
+strip_existential(_^In, Out) :-
+    strip_existential(In, Out).
+
+qualify(M, C, Goal, Calls) :-
+    M == C,
+    !,
+    Calls = Goal.
+qualify(M, _, Goal, M:Goal).
+
